#
#  YKLook... GUI Library (for Ruby)
#
#  Copyright (c) 2009-2010 YorozuKotowariYa Ltd.
#
if RUBY_VERSION[0..2] != '1.8'
	raise 'needRubyVersion1.8.x'
end

class YKLookError < StandardError
	def initialize( prm_msg, prm_info = nil )
		super( prm_msg )
		@yk_info = prm_info
	end
	def yks_make_message( prm_buf, prm_hdr = '' )
		prm_buf << (prm_hdr + 'ex_typ: ') << self.class().to_s() << "\n"
		prm_buf << (prm_hdr + 'ex_msg: ') << message()           << "\n"
		YKLookLog.yks_msg_r( backtrace(), (prm_hdr + 'ex_btr'), prm_buf )
		if @yk_info
			YKLookLog.yks_msg_r( @yk_info, (prm_hdr + 'ex_info'), prm_buf )
		end
	end
	def yk_detail()
		ary = []
		yks_make_message( ary )
		return ary.join(nil)
	end
	def yk_info()
		return @yk_info
	end
end
class YKLookErrorMain < YKLookError
end
class YKLookErrorSwing < YKLookError
end
class YKLookErrorUser < YKLookError
	def initialize( prm_usrex, prm_msg, prm_info = nil, prm_usrmsg = nil )
		super( prm_msg, prm_info )
		@yk_user_ex  = prm_usrex
		@yk_user_msg = prm_usrmsg
	end
	def yks_make_message( prm_buf, prm_hdr = '' )
		super( prm_buf, prm_hdr )
		YKLookLog.yks_msg_ex( @yk_user_ex, prm_buf, (prm_hdr + 'user_') )
	end
	def yk_user_ex()
		return @yk_user_ex
	end
	def yk_user_msg()
		return @yk_user_msg
	end
end
class YKLookLog
	YKLook_HdrFmt =
		"[%y-%m-%d %H:%M:%S - #{$$}]----------------------------------------\n"

	@@yk_log = nil
	@@yk_cwd = nil

	def self.yks_cwd()
		if @@yk_cwd
			return @@yk_cwd
		end
		@@yk_cwd = Dir.pwd()
	end
	def initialize( prm_path = nil, prm_size = 1048576 )
		super()
		@yk_dirname   = nil
		@yk_filename  = nil
		@yk_outpath   = nil
		@yk_sizelimit = 1048576
		yk_set_fullpath( prm_path )
		yk_set_sizelimit( prm_size )
	end
	def self.yk_log()
		if not @@yk_log
			@@yk_log = YKLookLog.new()
		end
		return @@yk_log
	end
	def yk_print( *prm_msg )
		wrk = YKLookLog.yks_msg( nil, prm_msg )
		yks_out( wrk )
		return wrk
	end
	def self.yk_print( *prm_msg )
		return yk_log().yk_print( *prm_msg )
	end
	def yk_print_exception( ex, *prm_msg )
		wrk = YKLookLog.yks_msg( ex, prm_msg )
		yks_out( wrk )
		return wrk
	end
	def self.yk_print_exception( ex, *prm_msg )
		return yk_log().yk_print_exception( ex, prm_msg )
	end
	def yk_fullpath()
		if @yk_outpath.nil?()
			yks_path()
		end
		return @yk_outpath
	end
	def yk_delete()
		yk_fullpath()
		if @yk_outpath
			if File.exist?( @yk_outpath )
				File.delete( @yk_outpath )
			end
		end
	end
	def self.yks_msg_r( prm_data, prm_leader, prm_buf )
		case prm_data
		when Array then
			prm_data.each_with_index() do |item,idx|
				yks_msg_r( item, prm_leader + "[#{idx}] ", prm_buf )
			end
		when Hash then
			prm_data.each() do |key,val|
				yks_msg_r( val, prm_leader + "[#{key}] ", prm_buf )
			end
		else
			if prm_data.respond_to?(:to_s)
				str = prm_data.to_s()
				if str.size() > 200
					str = sprintf( ' --- size=%d --- ', str.size() )
				end
			else
				str = sprintf( ' --- unknown data (%s) --- ',
								prm_data.class().to_s() )
			end
			prm_buf << prm_leader << str << "\n"
		end
	end
	def self.yks_msg_ex( prm_ex, prm_buf, prm_hdr = '' )
		if prm_ex.kind_of?( YKLookError )
			prm_ex.yks_make_message( prm_buf, prm_hdr )
		else
			prm_buf << (prm_hdr+'ex_typ: ') << prm_ex.class().to_s() << "\n"
			prm_buf << (prm_hdr+'ex_msg: ') << prm_ex.message()      << "\n"
			yks_msg_r( prm_ex.backtrace(), (prm_hdr+'ex_btr'), prm_buf )
		end
	end
	def self.yks_msg( prm_ex, prm_msg )
		res = [ Time.now().strftime( YKLook_HdrFmt ) ]
		if prm_ex
			yks_msg_r( caller()[1..-1], 'log_btr', res )
		end
		if prm_ex.kind_of?(Exception)
			yks_msg_ex( prm_ex, res, '' )
		end
		prm_msg.each do |item|
			cls = item.class().to_s()
			case item
				when Array then yks_msg_r( item, cls, res )
				when Hash  then yks_msg_r( item, cls, res )
				else res << cls << ': ' << item.to_s() << "\n"
			end
		end
		return res.join(nil)
	end
	def yks_out( prm_msg )
		begin
			yk_fullpath()
			if @yk_outpath
				if @yk_sizelimit.kind_of?(Integer)
					if File.exist?( @yk_outpath )
						if File.size( @yk_outpath ) > @yk_sizelimit
							File.rename( @yk_outpath, @yk_outpath + '_1' )
						end
					end
				end
				File.open( @yk_outpath, 'a' ) do |f|
					begin
						f.chmod( 0666 )
					rescue => ex
					end
					f.puts( prm_msg )
					f.flush()
					f.fsync()
				end
			else
				STDERR.puts( prm_msg )
			end
		rescue => ex
		end
	end
	def yks_path()
		if @yk_dirname
			dir = @yk_dirname
		else
			dir = YKLookLog.yk_path_find( 'YKLOOK_LOGPATH', 'yklooklog' )
			if not dir
				return
			end
		end
		if @yk_filename
			adr = @yk_filename
		else
			adr = ENV['REMOTE_ADDR']
		end
		if adr and adr != ''
			@yk_outpath = dir + '/' + adr
			return
		end
		tty = ''
		begin
			IO.popen('tty') do |io|
				tty = io.gets().strip().slice(5..-1)
			end
		rescue
		end
		if tty and tty != ''
			tty = tty.gsub(/\//,'_')
			@yk_outpath = dir + '/' + tty
			return
		end
		@yk_outpath = dir + '/tty'
	end
	def self.yk_path_find( prm_env, prm_name )
		dir = nil
		ary = [ ENV[prm_env], ENV['TEMP'], ENV['TMP'],
				'/var/tmp', '/usr/tmp', '/tmp', @@yk_cwd ]
		ary.each() do |e|
			if e and e != ''
				e = File.expand_path( e )
				if File.directory?( e )
					dir = e + ((e[-1,1] == '/') ? '' : '/') + prm_name
					if File.directory?( dir )
						if File.writable?( dir )
							break
						end
					else
						if File.writable?( e )
							Dir.mkdir( dir )
							File.chmod( 0777, dir )
							break
						end
					end
					dir = nil
				end
			end
		end
		if dir.nil?() or dir.empty?()
			return nil
		end
		[ 'YKLOOK_USER', 'USER', 'LOGNAME', 'USERNAME' ].each() do |u|
			e = ENV[u]
			if e and e != ''
				dir = dir + '/' + e
				if not File.directory?( dir )
					Dir.mkdir( dir )
					File.chmod( 0777, dir )
				end
				break
			end
		end
		return dir
	end
	def yks_check_string( prm_string )
		if not prm_string.kind_of?(String)
			raise YKLookError, 'notString'
		end
		if prm_string.empty?()
			raise YKLookError, 'emptyString'
		end
	end
	def yk_set_dirname( prm_dir )
		if prm_dir.nil?()
			@yk_dirname = nil
			return
		end
		yks_check_string( prm_dir )
		prm_dir = File.expand_path( prm_dir )
		if not File.directory?( prm_dir )
			Dir.mkdir( prm_dir )
			File.chmod( 0777, prm_dir )
		end
		@yk_dirname = prm_dir
	end
	def yk_set_filename( prm_file )
		if prm_file.nil?()
			@yk_filename = nil
			return
		end
		yks_check_string( prm_file )
		@yk_filename = File.basename( prm_file )
	end
	def yk_set_fullpath( prm_path )
		if prm_path.nil?()
			@yk_outpath = nil
			return
		end
		yks_check_string( prm_path )
		@yk_outpath = prm_path
	end
	def yk_set_sizelimit( prm_size )
		if prm_size.nil?()
			@yk_sizelimit = nil
			return
		end
		if not prm_size.kind_of?(Integer)
			raise YKLookError, 'notInteger'
		end
		if prm_size < 1024
			raise YKLookError, 'illInteger'
		end
		@yk_sizelimit = prm_size
	end
	def self.yk_sweep( prm_dir, prm_num, prm_unit, prm_max )
		if prm_dir.nil?()
			fp = yk_log().yk_fullpath()
			if not fp
				raise YKLookError, 'unknownSweepPath'
			end
			prm_dir = File.dirname( fp )
		end
		if not File.directory?( prm_dir )
			raise YKLookError, 'unknownSweepDir'
		end
		case prm_unit.upcase()
			when 'DAY'  then sec = 60 * 60 * 24
			when 'HOUR' then sec = 60 * 60
			when 'MIN'  then sec = 60
			else raise YKLookError, 'unknownSweepUnit'
		end
		ref = Time.now() - (sec * prm_num)
		cnt = 0
		Dir.foreach( prm_dir ) do |file|
			if file[0,1] != '.' and file[0,1] != '_'
				path = File.join( prm_dir, file )
				if File.writable?( path )
					tim = File.mtime( path )
					if tim < ref
						begin
							File.delete( path )
							cnt += 1
						rescue
						end
					end
				end
			end
			if cnt >= prm_max
				break
			end
		end
	end
end
class YKLookMain
	YKLookFar_jar    = 'yklookfar.jar'
	YKLookNear_jar   = 'yklooknear.jar'
	YKLook_Exit      = 'YKLook_Exit'
	YKLook_Return    = 'YKLook_Return'
	YKLook_mainpanel = 'yk_mainpanel'
	YKLook_maininfo  = 'yk_maininfo'
	YKLook_nearframe = 'yk_nearframe'

	def self.yks_stdio_redirect()
		@@yk_reddir = YKLookLog.yk_path_find( 'YKLOOK_REDPATH', 'yklookred' )
		if yk_is_far() and @@yk_reddir
			@@yk_redin  = yks_tmpfile( @@yk_reddir + "/p#{$$}-in" )
			@@yk_redout = yks_tmpfile( @@yk_reddir + "/p#{$$}-out" )
			@@yk_stdin  = STDIN .dup()
			@@yk_stdout = STDOUT.dup()
			STDIN .reopen( @@yk_redin  )
			STDOUT.reopen( @@yk_redout )
			$stdin  = STDIN
			$stdout = STDOUT
		end
	end
	def self.yks_tmpfile( prm_path )
		file = File.open( prm_path, 'w+' )
		begin
			File.delete( prm_path )
		rescue
		end
		return file
	end
	def self.yks_delete_redfile()
		if yk_is_far() and @@yk_reddir
			pi = @@yk_redin .path()
			po = @@yk_redout.path()
			begin
				@@yk_redin .close()
				@@yk_redout.close()
				File.delete( pi )
				File.delete( po )
			rescue
			end
		end
	end
	def self.yk_find_farpath()
		target = nil
		doc = ENV['DOCUMENT_ROOT']
		ary = [ doc, (doc + '/yklook'), '', 'yklook', '..', '../yklook' ]
		ary.each() do |dir|
			path = File.expand_path( dir, YKLookLog.yks_cwd() )
			path = path + ((path[-1,1] == '/') ? '' : '/') + YKLookFar_jar
			if File.exist?( path )
				target = dir
				break
			end
		end
		if target.nil?() or target.empty?()
			return nil
		end
		if target == doc
			return '/'
		end
		if target == (doc + '/yklook')
			return '/yklook'
		end
		return target
	end
	def yki_ht_farpath()
		far = ENV['YKLOOK_FARPATH']
		if not far
			far = YKLookMain.yk_find_farpath()
		end
		return [ far, nil ]
	end
	def self.yk_is_far()
		return (not yk_is_near())
	end
	def yk_is_far()
		return YKLookMain.yk_is_far()
	end
	def self.yk_is_near()
		return ( ENV['REMOTE_ADDR'].nil?() or ENV['REQUEST_METHOD'].nil?() )
	end
	def yk_is_near()
		return YKLookMain.yk_is_near()
	end
	def yk_is_frame()
		return yk_is_near()
	end
	def yk_is_mac()
		return YKLookMain.yk_is_mac()
	end
	def self.yk_is_mac()
		return RUBY_PLATFORM.include?( 'darwin' )
	end
	def yk_is_sun()
		return YKLookMain.yk_is_sun()
	end
	def self.yk_is_sun()
		return RUBY_PLATFORM.include?( 'solaris' )
	end
	def self.yk_mainloop( prm_main )
		@@yk_main = prm_main
		@@yk_framestack = [0]
		@@yk_framepool  = { 0 => prm_main }
		begin
			prm_main.yks_initialize()
			if yk_is_far()
				prm_main.yks_far()
			else
				prm_main.yks_near()
			end
		rescue => ex
			prm_main.yks_delete_selfdata()
			yks_delete_redfile()
			YKLookLog.yk_print_exception( ex, 'mainloop' )
			if ex.kind_of?(YKLookErrorUser)
				raise ex.yk_user_ex()
			end
			raise ex
		end
		yks_delete_redfile()
		prm_main.yki_sweep()
	end
	def self.yk_main()
		return @@yk_main
	end
	def yks_initialize()
		@yk_selfid   = nil
		@yk_userid   = nil
		@yk_yklsdata = { }
		@yk_instdata = []
	end
	def yks_set_far()
		@yk_in  = @@yk_stdin
		@yk_out = @@yk_stdout
	end
	def self.yk_find_nearpath()
		dir = yks_find_yklookpath()
		if File.exist?( dir + '/' + YKLookNear_jar )
			return dir
		end
		raise YKLookErrorMain.new( 'notFoundNearJar', { 'dir' => dir } )
	end
	def yki_sa_javapath()
		return 'java '
	end
	def yki_sa_javaoption()
		if YKLookMain.yk_is_mac()
			ext = File.extname( $0 )
			if ext.empty?()
				bn = File.basename( $0 )
			else
				bn = File.basename( $0, ext )
			end
			return (' -Xdock:name="' + bn + '" ')
		end
		return ''
	end
	def yki_sa_nearpath()
		return YKLookMain.yk_find_nearpath()
	end
	def yki_sa_javafull()
		jp = yki_sa_javapath()
		jo = yki_sa_javaoption()
		np = yki_sa_nearpath()
		return (jp + ' ' + jo +
				' -jar ' + np + '/' + YKLookNear_jar +
				' yks=' + @yk_selfid + ' yku=' + @yk_userid)
	end
	def yks_set_near()
		begin
			jf = yki_sa_javafull()
			yks_check_impl_string( jf, 'yki_sa_javafull' )
		rescue => ex
			yks_user_error( ex, 'yki_sa_javafull' )
		end
		begin
			@yk_in = IO.popen( jf, 'r+')
			@yk_in.binmode()
			@yk_out = @yk_in
		rescue => ex
			yks_user_error( ex, 'notExecJava', { 'cmd' => jf } )
		end
	end
	def yks_check_impl_string( prm_target, prm_method,
							prm_empty = nil, prm_quote = nil, prm_is_ary = nil )
		if not prm_target.kind_of?(String)
			if prm_is_ary
				raise YKLookErrorMain, ('implNotStrArray ' + prm_method)
			end
			raise YKLookErrorMain, ('implNotString ' + prm_method)
		end
		if prm_empty and prm_target.empty?()
			raise YKLookErrorMain, ('implEmptyString ' + prm_method)
		end
		if prm_quote and prm_target.include?('"')
			raise YKLookErrorMain, ('implIllQuote ' + prm_method)
		end
	end
	def yks_check_impl_array( prm_target, prm_method, prm_size = 2 )
		if not prm_target.kind_of?(Array)
			raise YKLookErrorMain, ('implNotArray ' + prm_method)
		end
		if prm_target.size() != prm_size
			raise YKLookErrorMain, ('implIllSizeArray ' + prm_method)
		end
	end
	def yks_check_impl_int( prm_target, prm_method )
		if not prm_target.kind_of?(Integer)
			raise YKLookErrorMain, ('implNotIntArray ' + prm_method)
		end
		if prm_target <= 0 or prm_target >=4096
			raise YKLookErrorMain, ('implIllNumber ' + prm_method)
		end
	end
	def yks_check_getdata()
		str = ENV['QUERY_STRING']
		if str
			ary = str.split('&')
			if ary.size() == 2
				s = ary[0]
				u = ary[1]
				if  s.length() >= 5 and s[0..3] == 'yks=' and
					u.length() >= 5 and u[0..3] == 'yku='
					return [ s[4..-1], u[4..-1] ]
				end
			end
		end
		return []
	end
	def yks_check_postdata( prm_size = nil )
		if prm_size
			str = @yk_in.read( prm_size )
		else
			str = @yk_in.read()
		end
		ary = str.split('&')
		if ary.size() != 3 and ary.size() != 4
			raise YKLookErrorMain, 'illPostDataCnt'
		end
		ary.each() do |item|
			if item.size() < 5 or item[3,1] != '='
				raise YKLookErrorMain, 'illPostDataFmt'
			end
		end
		s = ary[0][4..-1]
		u = ary[1][4..-1]
		g = ary[2][4..-1]
		if ary.size() == 3
			return [s, u, g]
		end
		e = ary[3][4..-1]
		return [s, u, g, e]
	end
	def yks_wait_postdata()
		begin
			buf = @yk_in.gets("\n")
			siz = buf.to_i()
		rescue
			siz = -1
		end
		return siz
	end
	def yks_check_args()
		wrk = {}
		ARGV.each() do |arg|
			if arg and arg.length() >= 6
				case arg[0..4]
					when '-yks=' then wrk['yks'] = arg[5..-1]
					when '-yku=' then wrk['yku'] = arg[5..-1]
				end
			end
		end
		return wrk
	end
	def yk_userargs( prm_head = nil )
		if prm_head
			if not prm_head.kind_of?(String)
				raise YKLookErrorMain, 'paramNotString'
			end
			len = prm_head.length()
		else
			len = 0
		end
		if yk_is_far()
			env = ENV['QUERY_STRING']
			if env and not env.empty?()
				ary = env.split('&')
			else
				ary = []
			end
		else
			ary = ARGV
		end
		wrk = []
		ary.each() do |arg|
			if not arg
				next
			end
			if arg.length() >= 2 and arg[0..1] == 'yk'
				next
			end
			if arg.length() >= 3 and arg[0..2] == '-yk'
				next
			end
			if arg.length() >= 4 and arg[0..3] == '--yk'
				next
			end
			if len == 0
				wrk << arg
				next
			end
			if len <= arg.length() and arg[0...len] == prm_head
				wrk << arg
			end
		end
		return wrk
	end
	def self.yks_encode_string( prm_data )
		if prm_data.nil?()
			return 'null'
		end
		str = prm_data.to_s()
		if str.empty?()
			return 'empty'
		end
		return str.unpack('H*')[0]
	end
	def self.yks_decode_string( prm_data )
		if prm_data.casecmp('empty') == 0
			return ''
		end
		if prm_data.casecmp('null') == 0
			return nil
		end
		return [prm_data].pack('H*')
	end
	def yks_decode_event( prm_event )
		ary = prm_event.split(':')
		@yk_event = Kernel.const_get( ary[0] ).new()
		@yk_event.yks_decode( ary )
	end
	def yks_decode_gui( prm_gui )
		if prm_gui.casecmp('empty') == 0
			return
		end
		ary = prm_gui.split(':')
		idx = 0
		while idx < ary.size()
			if ary[idx].empty?()
				idx += 1
				next
			end
			gui = YKLookSwingBase.yks_find_source( ary[idx].to_i() )
			idx += 1
			if gui
				idx = gui.yks_decode( ary, idx )
			end
		end
	end
	def yks_decode_gui_main( prm_gui )
		hash = { }
		ary  = prm_gui.split(':')
		1.step( ary.size()-1, 2 ) do |idx|
			if ary[idx] == 'end'
				break
			end
			case ary[idx][0..1]
				when 'S.' then val = YKLookMain.yks_decode_string( ary[idx+1] )
				when 'i.' then val = ary[idx+1].to_i()
				else           val = ary[idx+1]
			end
			hash[ ary[idx][2..-1] ] = val
		end
		@yk_yklsdata[ YKLook_maininfo ] = hash
	end
	def yk_maininfo( prm_key = nil )
		hash = @yk_yklsdata[ YKLook_maininfo ]
		if prm_key
			return hash[ prm_key ]
		end
		return hash
	end
	def yks_sespath()
		return YKLookLog.yk_path_find( 'YKLOOK_SESPATH', 'yklookses' )
	end
	def yks_load_selfdata( prm_selfid, prm_userid, prm_need_gui )
		@yk_selfid = prm_selfid
		@yk_userid = prm_userid
		dir = yks_sespath().to_s()
		gui = dir + '/s' + prm_selfid + 'S'
		wrk = dir + '/s' + prm_selfid + 'W'
		usr = dir + '/s' + prm_userid + 'U'
		begin
			if prm_need_gui and File.exist?( gui )
				f = File.open( gui, 'rb' )
				ary = Marshal.load( f )
				f.close()
				YKLookSwingBase.yks_load( ary[0] )
				yks_load_instdata( ary[1] )
				@yk_yklsdata = ary[2]
			else
				@yk_yklsdata = { }
				@yk_instdata = []
			end
			if File.exist?( wrk )
				f = File.open( wrk, 'rb' )
				@yk_userwork = Marshal.load( f )
				f.close()
			else
				@yk_userwork = []
			end
			if File.exist?( usr )
				f = File.open( usr, 'rb' )
				@yk_userdata = Marshal.load( f )
				f.close()
			else
				@yk_userdata = { }
			end
		rescue => ex
			YKLookLog.yk_print_exception( ex, dir, prm_selfid, prm_userid )
			yks_make_selfdata()
		end
	end
	def yks_load_instdata( prm_data )
		@yk_instdata = []
		prm_data.each() do |key,val|
			instance_variable_set( key, val )
			@yk_instdata << key
		end
	end
	def yks_save_selfdata( prm_gui =true, prm_selfid =nil, prm_userwork =nil )
		if not prm_selfid.kind_of?(String)
			prm_selfid = @yk_selfid
		end
		if not prm_userwork.kind_of?(Array)
			prm_userwork = @yk_userwork
		end
		dir = yks_sespath()
		if dir.nil?()
			return
		end
		gui = dir + '/s' + prm_selfid + 'S'
		wrk = dir + '/s' + prm_selfid + 'W'
		usr = dir + '/s' + @yk_userid + 'U'
		begin
			if prm_gui
				s = YKLookSwingBase.yks_save()
				i = yks_save_instdata()
				begin
					m = Marshal.dump( [s, i, @yk_yklsdata] )
				rescue => ex
					yks_user_error( ex, 'illDumpSys' )
				end
				begin
					f = File.open( gui, 'wb' )
					f.write( m )
					f.close()
				rescue => ex
					yks_user_error( ex, 'illSaveSys' )
				end
			end
			if prm_userwork.size() > 0 or File.exist?( wrk )
				begin
					m = Marshal.dump( prm_userwork )
				rescue => ex
					yks_user_error( ex, 'illDumpWrk' )
				end
				begin
					f = File.open( wrk, 'wb' )
					f.write( m )
					f.close()
				rescue => ex
					yks_user_error( ex, 'illSaveWrk' )
				end
			end
			if @yk_userdata.size() > 0 or File.exist?( usr )
				begin
					m = Marshal.dump( @yk_userdata )
				rescue => ex
					yks_user_error( ex, 'illDumpUsr' )
				end
				begin
					f = File.open( usr, 'wb' )
					f.write( m )
					f.close()
				rescue => ex
					yks_user_error( ex, 'illSaveUsr' )
				end
			end
		rescue => ex
			YKLookLog.yk_print_exception( ex, dir, prm_selfid,
													@yk_selfid, @yk_userid )
			raise ex
		end
	end
	def yks_save_instdata()
		hash = { }
		@yk_instdata.each() do |key|
			hash[key] = instance_variable_get(key)
		end
		return hash
	end
	def yks_delete_selfdata()
		if @yk_selfid
			dir = yks_sespath()
			if dir
				begin
					File.delete( dir + '/s' + @yk_selfid + 'S' )
				rescue
				end
				begin
					File.delete( dir + '/s' + @yk_selfid + 'W' )
				rescue
				end
				begin
					File.delete( dir + '/s' + @yk_selfid + 'U' ) # not userid
				rescue
				end
			end
		end
	end
	def yks_make_selfdata()
		@yk_selfid = YKLookMain.yks_more_id()
		@yk_userid = @yk_selfid
		@yk_userdata = { }
		@yk_userwork = []
		@yk_yklsdata = { }
		@yk_instdata = []
	end
	def self.yks_more_id
		a = ['a','b','c','d','e','f']
		n = YKLookSwingBase.yks_more_id()
		t = Time.now().to_i() & 0x0ffff
		i = [ a[ rand(6) ] ]
		i << rand(t).to_s(16) << a[ rand(6) ] << t  << a[ rand(6) ]
		i << n                << a[ rand(6) ] << $$ << a[ rand(6) ]
		return i.join(nil)
	end
	def self.yks_frame_current()
		@@yk_framestack[-1]
	end
	def yk_var( *prm_args )
		if prm_args.size() == 0
			return @yk_instdata
		end
		nam = prm_args[0]
		if not nam.kind_of?(String)
			raise YKLookErrorMain, 'notStringInArray'
		end
		if idx = nam.index('@')
			if idx != 0
				raise YKLookErrorMain, 'illNameFmt'
			end
		end
		if nam.index('@',1)
			raise YKLookErrorMain, 'illNameFmt'
		end
		YKLookMain.yks_check_yk( nam, 1 )
		tf = @yk_instdata.include?( nam )
		if not tf
			@yk_instdata << nam
		end
		if instance_variable_get( nam ).nil?()
			instance_variable_set( nam, nil )
		end
		if prm_args.size() == 2
			instance_variable_set( nam, prm_args[1] )
		end
		return tf
	end
	def yk_var_attach( prm_args )
		if not prm_args.kind_of?(Array)
			prm_args = [ prm_args ]
		end
		tf = false
		prm_args.each() do |key|
			if yk_var( key )
				tf = true
			end
		end
		return tf
	end
	def yk_var_detach( prm_args )
		if not prm_args.kind_of?(Array)
			prm_args = [ prm_args ]
		end
		tf = false
		prm_args.each() do |key|
			if @yk_instdata.include?( key )
				tf = true
				@yk_instdata.delete( key )
			end
		end
		return tf
	end
	def yk_userdata_del( prm_key = nil )
		if prm_key
			@yk_userdata.delete( prm_key )
		else
			@yk_userdata = { }
			dir = yks_sespath()
			begin
				File.delete( dir + '/s' + @yk_userid + 'U' )
			rescue
			end
		end
	end
	def self.yk_userdata_del()
		@@yk_main.yk_userdata_del()
	end
	def yk_userdata( *prm_keyval )
		if prm_keyval.size() == 0
			return @yk_userdata
		end
		if not prm_keyval[0].kind_of?(String)
			raise YKLookErrorMain, 'paramNotString'
		end
		if prm_keyval.size() == 1
			return @yk_userdata[ prm_keyval[0] ]
		end
		YKLookMain.yks_check_yk( prm_keyval[0] )
		@yk_userdata[ prm_keyval[0] ] = prm_keyval[1]
		return prm_keyval[1]
	end
	def self.yk_userdata( *prm_keyval )
		return @@yk_main.yk_userdata( *prm_keyval )
	end
	def self.yks_check_yk( prm_word, prm_idx = 0 )
		idx = prm_word.to_s().downcase().index('yk')
		if idx.kind_of?(Integer) and idx == prm_idx
			raise YKLookErrorMain, 'illNameFmt'
		end
	end
	def yk_userwork()
		return ((@yk_userwork.size() > 0) ? @yk_userwork[0] : nil)
	end
	def yk_html_escape( prm_msg, prm_chr = '' )
		msg = prm_msg.gsub( '&', '&amp;' )
		tmp =     msg.gsub( '<', '&lt;'  )
		wrk =     tmp.gsub( '>', '&gt;'  )
		prm_chr.each_byte() do |b|
			wrk = wrk.gsub( b.chr(), sprintf('&#%03d;',b) )
		end
		tmp =  wrk.gsub( "\r\n", '<br />' )
		wrk =  tmp.gsub( "\r"  , '<br />' )
		return wrk.gsub( "\n"  , '<br />' )
	end
	def yki_ht_error( prm_ex, prm_trigger, prm_info )
		return yk_html_escape( yki_error( prm_ex, prm_trigger, prm_info ) )
	end
	def yki_error( prm_ex, prm_trigger, prm_info )
		return 'Error ' + prm_trigger
	end
	def yks_user_error( prm_ex, prm_trigger, prm_info = {} )
		YKLookLog.yk_print_exception( prm_ex, prm_trigger, prm_info )
		if not prm_ex.kind_of?( YKLookErrorUser )
			begin
				msg = [ yki_error( prm_ex, prm_trigger, prm_info ) ,
						yki_ht_error( prm_ex, prm_trigger, prm_info ) ]
			rescue => ex
				msg = [ 'Error!', '<br><h1>Error!</h1>' ]
				prm_ex = ex
			end
			prm_ex = YKLookErrorUser.new( prm_ex, prm_trigger, prm_info, msg )
		end
		raise prm_ex
	end
	def self.yks_packet( prm_id, prm_method, prm_values = [] )
		@@yk_main.yks_packet( prm_id, prm_method, prm_values )
	end
	def yks_packet( prm_id, prm_method, prm_values = [] )
		ary = []
		prm_values.each() do |item|
			if item
				item = item.to_s()
				if item.empty?()
					ary << 'empty'
				else
					ary << item
				end
			else
				ary << 'null'
			end
		end
		@yk_packet << prm_id.to_s() + ':' + prm_method + ':'
		@yk_packet << ary.join(':') + ':end:'
	end
	def yki_fr_resizable()
		return true
	end
	def yki_fr_location()
		return [ nil, nil ]
	end
	def yki_fr_need_pack()
		return false
	end
	def yks_make_display_far()
		@yk_packet = []
		yks_set_mainpanel()
		catch( YKLook_Return ) do
			begin
				yki_make_display()
			rescue => ex
				yks_user_error( ex, 'yki_make_dispkay' )
			end
		end
	end
	def yks_make_display_near()
		@yk_packet = []
		yks_set_mainpanel()
		frm = @yk_yklsdata[ YKLook_nearframe ]
		ttl = yki_title()
		if ttl.kind_of?(String)
			yks_packet( frm, 'yk_setTitle',
						[ YKLookMain.yks_encode_string( ttl ) ] )
		end
		siz = yks_check_size()
		yks_packet( frm, 'yk_setSize', siz )
		res = yki_fr_resizable() ? 'T' : 'F'
		yks_packet( frm, 'yk_setResizable', [res] )
		loc = yki_fr_location()
		if loc.kind_of?(Array) and loc.size() == 2
			if loc[0].nil?() or loc[0].kind_of?(Integer)
				if loc[1].nil?() or loc[1].kind_of?(Integer)
					yks_packet( frm, 'yk_setLocation', loc )
				end
			end
		end
		catch( YKLook_Return ) do
			begin
				yki_make_display()
			rescue => ex
				yks_user_error( ex, 'yki_make_dispkay' )
			end
		end
		if yki_fr_need_pack()
			yks_packet( frm, 'yk_pack' )
		end
	end
	def yks_set_mainpanel( prm_frame_id = nil )
		panel = YKLookPanelQuint.new()
		@yk_yklsdata[ YKLook_mainpanel ] = panel
		yks_packet( 0, 'yk_setMainPanel', [panel.yk_id(), prm_frame_id] )
	end
	def yks_check_size()
		begin
			siz = yki_size()
			yks_check_impl_array( siz, 'yki_size' )
			yks_check_impl_int( siz[0], 'yki_size' )
			yks_check_impl_int( siz[1], 'yki_size' )
		rescue => ex
			yks_user_error( ex, 'yki_size' )
		end
		return siz
	end
	def yks_applet_size()
		siz = yks_check_size()
		return (' width=' + siz[0].to_s() + ' height=' + siz[1].to_s())
	end
	def yks_applet_path()
		begin
			far = yki_ht_farpath()
			yks_check_impl_array( far, 'yki_ht_farpath' )
			if not far[0].nil?()
				yks_check_impl_string(far[0], 'yki_ht_farpath', nil,true,true)
			end
			if not far[1].nil?()
				yks_check_impl_string(far[1], 'yki_ht_farpath', nil,true,true)
			end
			if far[0].nil?() or far[0].empty?()
				cb = ''
			else
				cb = ' codebase="' + far[0] + '"'
			end
			if far[1].nil?() or far[1].empty?()
				far[1] = ''
				ar = YKLookFar_jar
			else
				ar = far[1] + '/' + YKLookFar_jar
			end
		rescue => ex
			yks_user_error( ex, 'yki_ht_farpath' )
		end
		return [ (cb + ' archive="' + ar + '"'), far[1] ]
	end
	def yki_ht_space()
		return [nil, nil]
	end
	def yks_applet_space()
		begin
			spc = yki_ht_space()
			yks_check_impl_array( spc, 'yki_ht_space' )
			if spc[0]
				yks_check_impl_int( spc[0], 'yki_ht_space' )
				vs = ' vspace=' + spc[0].to_s()
			else
				vs = ''
			end
			if spc[1]
				yks_check_impl_int( spc[1], 'yki_ht_space' )
				hs = ' hspace=' + spc[1].to_s()
			else
				hs = ''
			end
		rescue => ex
			yks_user_error( ex, 'yki_ht_space' )
		end
		return (vs + hs)
	end
	def yki_ht_align()
		return nil
	end
	def yks_applet_align()
		begin
			ali = yki_ht_align()
			if ali
				yks_check_impl_string( ali, 'yki_ht_align', nil, true )
				return (' align=' + ali)
			end
		rescue => ex
			yks_user_error( ex, 'yki_ht_align' )
		end
		return ''
	end
	def yki_ht_name()
		return nil
	end
	def yks_applet_name()
		begin
			nam = yki_ht_name()
			if nam
				yks_check_impl_string( nam, 'yki_ht_name', true, true )
				return (' name=' + nam)
			end
		rescue => ex
			yks_user_error( ex, 'yki_ht_name' )
		end
		return ''
	end
	def yki_ht_alt()
		return 'YKLook -- Java Applet.'
	end
	def yks_applet_alt()
		begin
			alt = yki_ht_alt()
			if alt
				yks_check_impl_string( alt, 'yki_ht_alt' )
				return (' alt="' + alt.gsub('"',"'") + '"')
			end
		rescue => ex
			yks_user_error( ex, 'yki_ht_alt' )
		end
		return ''
	end
	def yki_ht_info()
		return yki_ht_alt()
	end
	def yks_applet_info()
		begin
			inf = yki_ht_info()
			if inf
				yks_check_impl_string( inf, 'yki_ht_info' )
				return inf
			end
		rescue => ex
			yks_user_error( ex, 'yki_ht_info' )
		end
		return ''
	end
	def yki_ht_border()
		return true
	end
	def yks_applet_border()
		begin
			return yki_ht_border()
		rescue => ex
			yks_user_error( ex, 'yki_ht_border' )
		end
	end
	def yki_ht_message_unknown()
		return yki_ht_alt()
	end
	def yks_applet_unknown()
		begin
			msg = yki_ht_message_unknown()
			if msg
				yks_check_impl_string( msg, 'yki_ht_message_unknown' )
				return msg
			end
		rescue => ex
			yks_user_error( ex, 'yki_ht_message_unknown' )
		end
		return ''
	end
	def yki_encoding()
		case $KCODE[0,1]
			when 'u','U' then enc = 'UTF8'
			when 's','S' then enc = 'SJIS'
			when 'e','E' then enc = 'EUC_JP'
			else              enc = 'ISO8859_1'
		end
		return enc
	end
	def yks_encoding()
		begin
			enc = yki_encoding()
			if enc
				yks_check_impl_string( enc, 'yki_encoding', nil, true )
				return enc
			end
		rescue => ex
			yks_user_error( ex, 'yki_encoding' )
		end
		return ''
	end
	def yki_laf_is_system()
		return true
	end
	def yki_laf()
		if yki_laf_is_system()
			return 'YKLook_laf_System'
		end
		return 'YKLook_laf_CrossPlatform'
	end
	def yks_laf()
		begin
			laf = yki_laf()
			if laf
				yks_check_impl_string( laf, 'yki_laf', nil, true )
				return laf.gsub( /\s/, '' )
			end
		rescue => ex
			yks_user_error( ex, 'yki_laf' )
		end
		return ''
	end
	def yks_param()
		return ''
	end
	def yk_ht_applet()
		siz = yks_applet_size()
		far = yks_applet_path()
		spc = yks_applet_space()
		ali = yks_applet_align()
		nam = yks_applet_name()
		alt = yks_applet_alt()
		own = File.basename( $0 )
		enc = yks_encoding()
		laf = yks_laf()
		inf = YKLookMain.yks_encode_string( yks_applet_info() )
		bdr = yks_applet_border() ? 'true' : 'false'
		unk = yks_applet_unknown()
		buf = []
		buf << '<applet' << siz << far[0]
		buf << ' code="jp.co.yorozukotowariya.yklook.YKLookApplet.class"'
		buf << spc << ali << nam << alt << ">\n"
		buf << '<param name=YKLook_relPath    value="' << far[1]     << "\">\n"
		buf << '<param name=YKLook_script     value="' << own        << "\">\n"
		buf << '<param name=YKLook_selfId     value="' << @yk_selfid << "\">\n"
		buf << '<param name=YKLook_userId     value="' << @yk_userid << "\">\n"
		buf << '<param name=YKLook_encoding   value="' << enc        << "\">\n"
		buf << '<param name=YKLook_laf        value="' << laf        << "\">\n"
		buf << '<param name=YKLook_appletInfo value="' << inf        << "\">\n"
		buf << '<param name=YKLook_border     value="' << bdr        << "\">\n"
		buf << yks_param()
		buf << unk
		buf << "\n</applet>\n"
		return buf.join(nil)
	end
	def yki_title()
		return nil
	end
	def yks_title()
		return yki_title()
	end
	def yki_ht_header()
		case $KCODE[0,1].downcase()
			when 'e' then cs = '; charset=EUC-JP'
			when 's' then cs = '; charset=Shift_JIS'
			when 'u' then cs = '; charset=UTF-8'
			else          cs = ''
		end
		return { 'Content-Type' => ('text/html' + cs) }
	end
	def yki_ht_html_header()
		buf = []
		yki_ht_header().each() do |key,val|
			buf<< '<meta http-equiv="' << key<< '" content="' << val<< "\">\n"
		end
		return buf.join(nil)
	end
	def yki_ht_html()
		ttl = yks_title()
		if ttl.kind_of?(String)
			ttl = '<title>' + yk_html_escape(ttl) + "</title>\n"
		else
			ttl = ''
		end
		return ("<html>\n<head>\n" + yki_ht_html_header() + ttl +
				"</head>\n<body>\n" + yk_ht_applet() + "</body>\n</html>\n")
	end
	def yki_ht_http_header()
		buf = []
		yki_ht_header().each() do |key,val|
			buf << key << ': ' << val << "\n"
		end
		return buf.join(nil)
	end
	def yki_ht_http()
		return ( yki_ht_http_header() + "\n" + yki_ht_html() )
	end
	def yks_send_command_far()
		begin
			buf = yki_ht_http()
		rescue => ex
			yks_user_error( ex, 'yki_ht_http' )
		end
		@yk_out.puts( buf )
	end
	def yks_make_command_near()
		path = yki_sa_nearpath()
		enc  = yks_encoding()
		laf  = yks_laf()
		frm  = YKLookSwingBase.yks_more_id()
		@yk_yklsdata[ YKLook_nearframe ] = frm
		buf = []
		buf << 'YKLook_nearPath='  << path                << "\n"
		buf << 'YKLook_encoding='  << enc                 << "\n"
		buf << 'YKLook_laf='       << laf                 << "\n"
		buf << 'YKLook_cwd='       << YKLookLog.yks_cwd() << "\n"
		buf << 'YKLook_nearFrame=' << frm                 << "\n"
		buf << yks_param()
		buf << "yk_end\n"
		return buf.join(nil)
	end
	def yks_send_command_near()
		buf = yks_make_command_near()
		len = buf.size().to_s() + "\n"
		@yk_out.write( len + buf )
		@yk_out.flush()
	end
	def yks_make_answer()
		@yk_packet = []
		begin
			catch( YKLook_Return ) do
				yki_action_any( @yk_event )
			end
		rescue => ex
			yks_user_error( ex, 'yki_action_any',
							{ 'yk_event' => @yk_event } )
		end
	end
	def yks_send_answer_far()
		buf = "Content-type: text/plain\n\nyklook-answer\n" +
				@yk_packet.join(nil) + "\n"
		@yk_out.write( buf )
	end
	def yks_send_answer_near()
		buf = "yklook-answer\n" + @yk_packet.join(nil) + "\n"
		len = buf.size().to_s() + "\n"
		@yk_out.write( len + buf )
		@yk_out.flush()
	end
	def yki_ht_error_html( prm_ex )
		if prm_ex.kind_of?(YKLookErrorUser)
			ary = prm_ex.yk_user_msg()
		else
			ary = [ nil, "<h1>Error!!</h1>\n" ]
		end
		return ("<html><body>\n" + ary[1] + "\n</body></html>\n")
	end
	def yki_ht_error_http( prm_ex )
		buf = yki_ht_error_html( prm_ex )
		return ("Content-type: text/html\n\n" + buf)
	end
	def yks_send_error_command_far( prm_ex )
		buf = nil
		begin
			buf = yki_ht_error_http( prm_ex )
		rescue => ex
			YKLookLog.yk_print_exception( ex )
			buf = "Content-type: text/html\n\n" +
					"<html><body>\nError!!!\n</body></html>\n"
		end
		@yk_out.puts( buf )
	end
	def yks_send_error_answer_far( prm_ex )
		if prm_ex.kind_of?(YKLookErrorUser)
			ary = prm_ex.yk_user_msg()
		else
			ary = [ 'Error!!', nil ]
		end
		msg = YKLookMain.yks_encode_string( ary[0] )
		buf = "Content-type: text/plain\n\nyklook-error\n" + msg
		@yk_out.write( buf )
	end
	def yks_far()
		yks_set_far()
		if ENV['REQUEST_METHOD'] != 'POST'
			begin
				ary = yks_check_getdata()
				if ary.size() >= 2
					yks_load_selfdata( ary[0], ary[1], false )
				else
					yks_make_selfdata()
				end
				yks_send_command_far()
				yks_save_selfdata()
			rescue => ex
				yks_delete_selfdata()
				YKLookLog.yk_print_exception( ex, 'far get' )
				yks_send_error_command_far( ex )
			end
		else
			begin
				ary = yks_check_postdata()
				if ary.size() == 3
					yks_load_selfdata( ary[0], ary[1], false )
					yks_decode_gui_main( ary[2] )
					sts = catch( YKLook_Exit ) do
						yks_make_display_far()
						nil
					end
				else
					yks_load_selfdata( ary[0], ary[1], true )
					yks_decode_gui( ary[2] )
					yks_decode_event( ary[3] )
					sts = catch( YKLook_Exit ) do
						yks_make_answer()
						nil
					end
				end
				yks_send_answer_far()
				if sts.nil?() or sts[0]
					yks_save_selfdata()
				else
					yks_delete_selfdata()
				end
			rescue => ex
				yks_delete_selfdata()
				YKLookLog.yk_print_exception( ex, 'far post' )
				yks_send_error_answer_far( ex )
			end
		end
	end
	def yks_near()
		begin
			h = yks_check_args()
			s = h['yks']
			u = h['yku']
			if s and u
				yks_load_selfdata( s, u, false )
			else
				yks_make_selfdata()
			end
			yks_set_near()
			sts = catch( YKLook_Exit ) do
				yks_send_command_near()
				while true
					siz = yks_wait_postdata()
					if siz <= 0
						yks_close_near( 'break' )
						break
					end
					ary = yks_check_postdata( siz )
					if ary.size() == 3
						yks_decode_gui_main( ary[2] )
						yks_make_display_near()
					else
						yks_decode_gui( ary[2] )
						yks_decode_event( ary[3] )
						yks_make_answer()
					end
					yks_send_answer_near()
					yki_sweep()
				end
				nil
			end
			if sts.kind_of?(Array) and sts[0]
				yks_save_selfdata()
			else
				yks_delete_selfdata()
			end
		rescue => ex
			yks_delete_selfdata()
			YKLookLog.yk_print_exception( ex, 'near' )
			raise ex
		end
	end
	def yk_mainpanel()
		return @yk_yklsdata[ YKLook_mainpanel ]
	end
	def yk_add( prm_gui, *prm_pos )
		yk_mainpanel().yk_add( prm_gui, *prm_pos )
	end
	def yki_sa_close( prm_cond )
		yk_exit()
	end
	def yks_close_near( prm_cond )
		begin
			yki_sa_close( prm_cond )
		rescue => ex
			yks_user_error( ex, 'yki_sa_close' )
		end
	end
	def yki_size()
		return [400, 300]
	end
	def yki_make_display()
		yk_add( YKLookLabel.new('YKLook') )
	end
	def yki_action_any( prm_event )
		return yk_action_callback( prm_event )
	end
	def yk_action_callback( prm_event )
		cb = prm_event.yk_callback()
		if not cb.kind_of?(String) or cb.empty?()
			return false
		end
		begin
			__send__( cb, prm_event )
		rescue => ex
			yks_user_error( ex, 'event_callback (' + cb + ')',
								{ 'cb' => cb, 'yks_event' => prm_event } )
		end
		return true
	end
	def yks_callback_near( prm_event )
		if prm_event.yk_answer() == 'Closing'
			yks_close_near( 'Closing' )
		else
			begin
				yki_sa_callback( prm_event )
			rescue => ex
				yks_user_error( ex, 'yki_sa_callback' )
			end
		end
	end
	def yki_sa_callback( prm_event )
	end
	def yk_set_default_button( prm_btn )
		yks_packet( 0, 'yk_setDefaultButton',
					[ (prm_btn ? prm_btn.yk_id() : nil) ] )
	end
	def yk_show_dialog_message( prm_cb, prm_gui, prm_title = nil,
								prm_type = nil, prm_icon = nil )
		yks_dialog_cb(      prm_cb    )
		yks_dialog_gui(     prm_gui   )
		yks_dialog_title(   prm_title )
		yks_dialog_msgtype( prm_type  )
		yks_dialog_icon(    prm_icon  )
		yks_packet( 0, 'yk_showMessageDialog' )
		yk_return()
	end
	def yk_show_dialog_confirm( prm_cb, prm_gui, prm_title = nil,
								prm_opttype = nil,
								prm_msgtype = nil, prm_icon = nil )
		yks_dialog_cb(      prm_cb    )
		yks_dialog_gui(     prm_gui   )
		yks_dialog_title(   prm_title )
		yks_dialog_opttype( prm_opttype  )
		yks_dialog_msgtype( prm_msgtype  )
		yks_dialog_icon(    prm_icon  )
		yks_packet( 0, 'yk_showConfirmDialog' )
		yk_return()
	end
	def yk_show_dialog_option( prm_cb, prm_gui, prm_title,
								prm_opt, prm_def = nil,
								prm_type = nil, prm_icon = nil )
		yks_dialog_cb(      prm_cb    )
		yks_dialog_gui(     prm_gui   )
		yks_dialog_title(   prm_title )
		yks_dialog_option(  prm_opt   )
		yks_dialog_default( prm_def   )
		yks_dialog_msgtype( prm_type  )
		yks_dialog_icon(    prm_icon  )
		yks_packet( 0, 'yk_showOptionDialog' )
		yk_return()
	end
	def yk_show_dialog_input( prm_cb, prm_gui, prm_title = nil,
								prm_type = nil, prm_icon = nil )
		yks_dialog_cb(      prm_cb    )
		yks_dialog_gui(     prm_gui   )
		yks_dialog_title(   prm_title )
		yks_dialog_msgtype( prm_type  )
		yks_dialog_icon(    prm_icon  )
		yks_packet( 0, 'yk_showInputDialog01' )
		yk_return()
	end
	def yk_show_dialog_select( prm_cb, prm_gui, prm_title,
								prm_sel, prm_def = nil,
								prm_type = nil, prm_icon = nil )
		yks_dialog_cb(      prm_cb    )
		yks_dialog_gui(     prm_gui   )
		yks_dialog_title(   prm_title )
		yks_dialog_option(  prm_sel   )
		yks_dialog_default( prm_def   )
		yks_dialog_msgtype( prm_type  )
		yks_dialog_icon(    prm_icon  )
		yks_packet( 0, 'yk_showInputDialog02' )
		yk_return()
	end
	def yks_dialog_cb( prm_cb )
		fg = ''
		if prm_cb.kind_of?(Array)
			if prm_cb.size() >= 1
				prm_cb = prm_cb[0]
				fg = 'all'
			else
				prm_cb = ''
			end
		end
		if prm_cb
			YKLookMain.yks_check_yk( prm_cb )
		end
		cb = YKLookMain.yks_encode_string( prm_cb )
		yks_packet( 0, 'yk_setDialogCB',
					[ cb, fg, YKLookMain.yks_frame_current() ] )
	end
	def yks_encode_any( prm_gui )
		if not prm_gui.kind_of?(Array)
			prm_gui = [ prm_gui ]
		end
		ary = [ prm_gui.size() ]
		prm_gui.each() do |obj|
			case obj
				when String, Symbol then
					ary << ('S/' + YKLookMain.yks_encode_string( obj ))
				when Integer         then ary << ('i/' + obj.to_s())
				when Float           then ary << ('f/' + obj.to_s())
				when NilClass        then ary <<  'n/null'
				when TrueClass       then ary <<  's/true'
				when FalseClass      then ary <<  's/false'
				when YKLookSwingBase then ary << ('o/' + obj.yk_id().to_s())
				else raise YKLookErrorMain, 'illParamObject'
			end
		end
		return ary
	end
	def yks_dialog_gui( prm_gui )
		yks_packet( 0, 'yk_setDialogGui', yks_encode_any( prm_gui ) )
	end
	def yks_dialog_title( prm_title )
		yks_packet( 0, 'yk_setDialogTitle',
					[ YKLookMain.yks_encode_string( prm_title ) ] )
	end
	def yks_dialog_msgtype( prm_type )
		yks_packet( 0, 'yk_setDialogMsgType', [ prm_type ] )
	end
	def yks_dialog_opttype( prm_type )
		yks_packet( 0, 'yk_setDialogOptType', [ prm_type ] )
	end
	def yks_dialog_icon( prm_icon )
		i = prm_icon.kind_of?(YKLookImage) ? prm_icon.yk_id() : 'null'
		yks_packet( 0, 'yk_setDialogIcon', [ i ] )
	end
	def yks_dialog_option( prm_opt )
		ary = [ prm_opt.size() ]
		prm_opt.each() do |opt|
			ary << YKLookMain.yks_encode_string( opt )
		end
		yks_packet( 0, 'yk_setDialogOption', ary )
	end
	def yks_dialog_default( prm_def )
		if prm_def.kind_of?(Integer)
			d = 'i/' + prm_def.to_s()
		else
			d = 'S/' + YKLookMain.yks_encode_string( prm_def )
		end
		yks_packet( 0, 'yk_setDialogDefault', [ d ] )
	end
	def yki_sweep()
		yk_sweep()
	end
	def yki_sweep_info()
		return [ 1, 'day', 1, 3, 'day', 1 ]
	end
	def yk_sweep()
		info = yki_sweep_info()
		path = yks_sespath()
		begin
			if @@yk_reddir
				YKLookLog.yk_sweep( @@yk_reddir, info[0], info[1], info[2] )
			end
		rescue
		end
		begin
			if path
				YKLookLog.yk_sweep( path, info[0], info[1], info[2] )
			end
		rescue
		end
		begin
			YKLookLog.yk_sweep( nil, info[3], info[4], info[5] )
		rescue
		end
	end
	def yki_sa_command_path( prm_path )
		return 'ruby'
	end
	def yki_sa_command_option( prm_path )
		return []
	end
	def yks_exec( prm_path, prm_selfid )
		cmd = yki_sa_command_path( prm_path )
		opt = yki_sa_command_option( prm_path )
		opt << prm_path
		opt << '-yks=' + prm_selfid
		opt << '-yku=' + @yk_userid
		kcd = '-K' + $KCODE[0,1].downcase()
		exec( cmd, kcd, *opt )
	end
	def yk_return()
		throw( YKLook_Return, nil )
	end
	def yk_next_display( prm_path, prm_info = nil )
		case prm_info
			when NilClass  then @yk_userwork = []
			when Array     then @yk_userwork = [ prm_info ]
			else raise YKLookErrorMain, 'illParamObject'
		end
		yks_save_selfdata()
		if yk_is_far()
			yks_packet( 0, 'yk_nextDisplay', prm_path.unpack('H*') )
			yk_return()
		else
			begin
				@yk_out.close()
				@yk_in.close()
			rescue => ex
			end
			yks_exec( prm_path, @yk_selfid )
		end
	end
	def yk_ht_more_display( *prm_path ) # [0]:path, [1]:info, [2]:share
		info = (prm_path.size() < 2) ? [] : [ prm_path[1] ]
		moreid = YKLookMain.yks_more_id()
		yks_save_selfdata( false, moreid, info )
		ary = prm_path[0].unpack('H*')
		ary << moreid
		ary << ((prm_path.size() < 3) ? 'F' : (prm_path[2] ? 'T' : 'F'))
		yks_packet( 0, 'yk_moreDisplay', ary )
	end
	def yk_ht_show_document( prm_url, prm_file = nil, prm_target = nil )
		ary = YKLookMain.yks_encode_url( prm_url, prm_file )
		ary << prm_target
		yks_packet( 0, 'yk_showDocument', ary )
		# @yk_userdata = { }
		# @yk_userwork = []
		# yks_delete_selfdata()
		yk_return()
	end
	def self.yks_encode_url( prm_url, prm_file )
		if not prm_url.kind_of?(String)
			prm_url = ''
		end
		if not prm_file.kind_of?(String)
			prm_file = ''
		end
		url  = yks_encode_string( prm_url )
		file = yks_encode_string( prm_file )
		return [ url, file ]
	end
	def yk_ht_exit( prm_msg = nil )
		if (not prm_msg.kind_of?(String)) or prm_msg.empty?()
			prm_msg = 'OK, Success.'
		end
		yks_packet( 0, 'yk_exit', prm_msg.unpack('H*') )
	end
	def yki_ht_exit( prm_msg )
		yk_ht_exit( prm_msg )
	end
	def yki_sa_exit( prm_msg )
		print( prm_msg )
	end
	def yk_exit( prm_msg = '' )
		if yk_is_far()
			yki_ht_exit( prm_msg )
		else
			yki_sa_exit( prm_msg )
		end
		throw( YKLook_Exit, [false] )
	end
	def yk_find_objects( prm_name = nil, prm_no = nil )
		return YKLookSwingBase.yks_find_objects( prm_name, prm_no )
	end
	def self.yk_find_objects( prm_name = nil , prm_no = nil )
		return YKLookSwingBase.yks_find_objects( prm_name, prm_no )
	end
	def yk_look( prm_name = nil, prm_no = nil )
		YKLookMain.yk_look( prm_name, prm_no )
	end
	def self.yk_look( prm_name = nil, prm_no = nil )
		ary = YKLookSwingBase.yks_find_objects( prm_name, prm_no )
		if ary.size() < 1
			return nil
		end
		return ary[0]
	end
	def self.yks_find_yklookpath()
		target = nil
		$LOADED_FEATURES.each() do |path|
			if path and path.length() > 15
				if path[-15..-1] == 'yklook/swing.rb'
					target = path
					break
				end
			end
		end
		if target.nil?()
			raise YKLookErrorMain, 'notLoad'
		end
		return target[0..-10]
	end
end
class YKLookSwingBase
	@@yk_id_current = 0
	@@yk_objects    = { }

	def self.yks_more_id()
		@@yk_id_current += 1
	end
	def self.yks_save()
		return [ @@yk_id_current, @@yk_objects ]
	end
	def self.yks_load( prm_data )
		@@yk_id_current = prm_data[0]
		@@yk_objects    = prm_data[1]
	end
	def initialize()
		super()
		@yk_frameid = YKLookMain.yks_frame_current()
		@yk_tagname = nil
		@yk_tagno   = 0
		@yk_prop    = { }

		@@yk_id_current += 1
		@yk_id = @@yk_id_current

		cls = self.class().to_s()
		ary = @@yk_objects[ cls ]
		if ary.nil?()
			ary = []
			@@yk_objects[ cls ] = ary
		end
		ary << self
		YKLookMain.yks_packet( 0, 'yk_new',
								[self.class().to_s(), @yk_frameid, @yk_id] )
	end
	def yk_var( prm_name )
		YKLookMain.yk_main().yk_var( prm_name, self )
		return self
	end
	def yk_id()
		return @yk_id
	end
	def yk_tag()
		return [@yk_tagname, @yk_tagno]
	end
	def yk_tagname()
		return @yk_tagname
	end
	def yk_tagno()
		return @yk_tagno
	end
	def yk_set_tag( prm_name, prm_no = 0 )
		if prm_name
			YKLookMain.yks_check_yk( prm_name )
		end
		@yk_tagname = prm_name
		@yk_tagno   = prm_no
		return self
	end
	def self.yk_look( prm_name = nil, prm_no = nil )
		ary = @@yk_objects[ self.to_s() ]
		if not ary.kind_of?(Array)
			return nil
		end
		target = nil
		ary.each() do |obj|
			if prm_name
				if prm_name.kind_of?(String)
					if obj.yk_tagname() != prm_name
						next
					end
				else
					if obj.yk_tagname().match( prm_name ).nil?()
						next
					end
				end
			end
			if prm_no
				if obj.yk_tagno() != prm_no
					next
				end
			end
			target = obj
			break
		end
		return target
	end
	def self.yk_find_objects( prm_name = nil, prm_no = nil )
		ary = @@yk_objects[ self.to_s() ]
		if not ary.kind_of?(Array)
			return []
		end
		target = []
		ary.each() do |obj|
			if prm_name
				if prm_name.kind_of?(String)
					if obj.yk_tagname() != prm_name
						next
					end
				else
					if obj.yk_tagname().match( prm_name ).nil?()
						next
					end
				end
			end
			if prm_no
				if obj.yk_tagno() != prm_no
					next
				end
			end
			target << obj
		end
		return target
	end
	def self.yks_find_objects( prm_name = nil, prm_no = nil )
		res = []
		@@yk_objects.each_value() do |ary|
			ary.each() do |obj|
				if prm_name
					if prm_name.kind_of?(String)
						if obj.yk_tagname() != prm_name
							next
						end
					else
						if obj.yk_tagname().match( prm_name ).nil?()
							next
						end
					end
				end
				if prm_no
					if obj.yk_tagno() != prm_no
						next
					end
				end
				res << obj
			end
		end
		return res
	end
	def self.yks_find_source( prm_id )
		@@yk_objects.each_value() do |ary|
			ary.each() do |item|
				if item.yk_id() == prm_id
					return item
				end
			end
		end
		return nil
	end
	def yks_check_gui( prm_gui )
		if not prm_gui.kind_of?(YKLookComponent)
			if not prm_gui.kind_of?(YKLookBoxFiller)
				raise YKLookErrorSwing, 'needGUI'
			end
		end
	end
	def yks_rpc( prm_method, *prm_values )
		YKLookMain.yks_packet( @yk_id, prm_method, prm_values );
	end
	def yks_rpca( prm_method, prm_values )
		YKLookMain.yks_packet( @yk_id, prm_method, prm_values );
	end
	def yk_set_tooltip( prm_text )
		yks_rpc( 'yk_setToolTipText', YKLookMain.yks_encode_string( prm_text ) )
	end
	def yk_set_listener( prm_lis )
		yks_rpc( 'yk_setListener', yks_check_listener( prm_lis ) )
		return self
	end
	def yks_check_listener( prm_lis )
		if prm_lis
			YKLookMain.yks_check_yk( prm_lis )
		end
		return YKLookMain.yks_encode_string( prm_lis )
	end
	def yk_put_property( prm_key, prm_val )
		YKLookMain.yks_check_yk( prm_key )
		@yk_prop[ prm_key ] = prm_val
	end
	def yk_get_property( prm_key = nil )
		if prm_key.nil?()
			return @yk_prop.keys()
		end
		return @yk_prop[ prm_key ]
	end
	def yk_get_prop_obj( prm_key )
		id = @yk_prop[ prm_key ]
		if id
			return YKLookSwingBase.yks_find_source( id.to_i() )
		end
		return nil
	end
	def yks_encode_color( prm_data )
		if prm_data.kind_of?(YKLookColor)
			return ( 'o/' + prm_data.yk_id().to_s() )
		end
		if prm_data.kind_of?(String)
			if prm_data[0,1] == '#'
				if prm_data.size() == 7 or prm_data.size() == 9
					return ('h/' + prm_data[1..-1])
				end
			end
			return ('s/' + prm_data)
		end
		if prm_data.nil?()
			return 'n/null'
		end
		col = prm_data.kind_of?(Array) ? prm_data : [ prm_data ]
		fst = col[0]
		case fst
			when Float       then m = 'f/'
			when Integer     then m = 'i/'
			when YKLookColor then m = 'b/' ; fst = fst.yk_id()
			else raise YKLookErrorSwing, 'illColor'
		end
		case col.size()
			when 1 then r = (fst >> 16) & 0x00ff
						g = (fst >>  8) & 0x00ff
                        b = (fst      ) & 0x00ff
						a = 255;
			when 2 then r = (fst >> 16) & 0x00ff
						g = (fst >>  8) & 0x00ff
                        b = (fst      ) & 0x00ff
						a =  col[1]
			when 3 then r = fst
						g = col[1]
						b = col[2]
						if m == 'f/'
							a = 1.0
						else
							a = 255
						end
			when 4 then r = fst
						g = col[1]
						b = col[2]
						a = col[3]
			else raise YKLookErrorSwing, 'illColor'
		end
		return (m + r.to_s() + '/' + g.to_s() + '/' + b.to_s() + '/' + a.to_s())
	end
	def yks_decode_trail( prm_ary, prm_idx )
		while( prm_ary[prm_idx] != 'end' )
			prm_idx += 1
		end
		return (prm_idx + 1)
	end
end
class YKLookComponent < YKLookSwingBase
	def initialize()
		super()
		@yk_prop['yk_enabled'] = true
		@yk_prop['yk_visible'] = true
	end
	def yks_decode( prm_ary, prm_idx )
		return yks_decode_trail( prm_ary, prm_idx )
	end
	def yk_text()
		return @yk_prop['yk_text']
	end
	def yk_icon()
		return yk_get_prop_obj( 'yk_icon' )
	end
	def yk_set_label( *prm_data )  # label,button,toggle,checkbox,radio
		prm_data.each() do |prm|
			if prm.kind_of?(String)
				yk_set_text( prm )
			end
			if prm.kind_of?(YKLookImage)
				yk_set_icon( prm )
			end
			if prm.kind_of?(TrueClass)
				yk_set_selected( true )
			end
			if prm.kind_of?(FalseClass)
				yk_set_selected( false )
			end
			if prm.kind_of?(Integer)
				yk_set_textposition( prm )
			end
		end
	end
	def yk_set_text( prm_text )  # label,button
		@yk_prop['yk_text'] = prm_text
		yks_rpc( 'yk_setText', YKLookMain.yks_encode_string( prm_text ) )
	end
	def yk_set_icon( prm_image )  # label,button
		@yk_prop['yk_icon'] = prm_image.kind_of?(YKLookImage) ?
								prm_image.yk_id() : nil
		yks_rpc( 'yk_setIcon', @yk_prop['yk_icon'] )
	end
	def yk_is_selected()
		return yks_get_bool( 'yk_selected' )
	end
	def yk_set_selected( prm_flag )
		yks_rpc( 'yk_setSelected', yks_set_bool('yk_selected', prm_flag) )
	end
	def yks_get_bool( prm_key )
		return (@yk_prop[prm_key] ? true : false)
	end
	def yks_set_bool( prm_key, prm_bool )
		@yk_prop[ prm_key ] = prm_bool
		return (prm_bool ? 'true' : 'false')
	end
	def yk_set_alignment_x( prm_percent )
		yks_rpc( 'yk_setAlignmentX', prm_percent );
	end
	def yk_set_alignment_y( prm_percent )
		yks_rpc( 'yk_setAlignmentY', prm_percent );
	end
	def yk_set_vertical_alignment( prm_align )
		yks_rpc( 'yk_setVerticalAlignment', prm_align );
	end
	def yk_set_horizontal_alignment( prm_align )
		yks_rpc( 'yk_setHorizontalAlignment', prm_align );
	end
	def yk_set_vertical_text_position( prm_pos )
		yks_rpc( 'yk_setVerticalTextPosition', prm_pos );
	end
	def yk_set_horizontal_text_position( prm_pos )
		yks_rpc( 'yk_setHorizontalTextPosition', prm_pos );
	end
	def yk_set_text_position( prm_pos )
		yks_rpc( 'yk_setTextPosition', prm_pos );
	end
	def yk_set_minimum_size( prm_width, prm_height = nil )
		yks_rpca( 'yk_setMinimumSize', yks_size( prm_width, prm_height ) )
	end
	def yk_set_maximum_size( prm_width, prm_height = nil )
		yks_rpca( 'yk_setMaximumSize', yks_size( prm_width, prm_height ) )
	end
	def yk_set_preferred_size( prm_width, prm_height = nil )
		yks_rpca( 'yk_setPreferredSize', yks_size( prm_width, prm_height ) )
	end
	def yks_size( prm_width, prm_height )
		if prm_width.kind_of?(Array)
			ary = [ prm_width[0], prm_width[1] ]
			if prm_height.kind_of?(Array)
				ary << prm_height[0].to_i()
				ary << prm_height[1].to_i()
			else
				ary << '0'
				ary << '0'
			end
			return ary
		end
		return [ prm_width.to_i(), prm_height.to_i(), '0', '0' ]
	end
	def yk_is_enabled()
		return yks_get_bool( 'yk_enabled' )
	end
	def yk_set_enabled( prm_flag )
		yks_rpc( 'yk_setEnabled', yks_set_bool('yk_enabled', prm_flag) )
	end
	def yk_is_visible()
		return yks_get_bool( 'yk_visible' )
	end
	def yk_set_visible( prm_flag )
		yks_rpc( 'yk_setVisible', yks_set_bool('yk_visible', prm_flag) )
	end
	def yk_font()
		return yk_get_prop_obj( 'yk_font' )
	end
	def yk_set_font( prm_font )
		@yk_prop['yk_font'] = prm_font.kind_of?(YKLookFont) ?
								prm_font.yk_id() : nil
		yks_rpc( 'yk_setFont', @yk_prop['yk_font'] )
	end
	def yk_fore()
		return yk_get_property( 'yk_fore' )
	end
	def yk_set_fore( prm_fore )
		@yk_prop['yk_fore'] = prm_fore
		yks_rpc( 'yk_setForeground', yks_encode_color( prm_fore ) )
	end
	def yk_back()
		return yk_get_property( 'yk_back' )
	end
	def yk_set_back( prm_back )
		@yk_prop['yk_back'] = prm_back
		yks_rpc( 'yk_setBackground', yks_encode_color( prm_back ) )
	end
	def yk_set_border( prm_bdr = nil, prm_ttl = nil )
		if not prm_bdr.kind_of?(YKLookBorderBase)
			prm_bdr = YKLookBorder.new( prm_bdr )
		end
		if prm_ttl
			prm_bdr = YKLookBorderTitled.new( prm_bdr, prm_ttl )
		end
		yks_rpc( 'yk_setBorder', prm_bdr.yk_id() )
	end
	def yk_set_cursor( prm_cursor )
		yks_set_cursor( prm_cursor, 'yk_setCursor' )
	end
	def yks_set_cursor( prm_cursor, prm_m )
		if prm_cursor.kind_of?(YKLookCursor)
			yks_rpc( prm_m, 'o/' + prm_cursor.yk_id().to_s() )
		else
			yks_rpc( prm_m, 's/' + prm_cursor.to_s() )
		end
	end
	def yk_set_opaque( prm_bool )
		yks_rpc( 'yk_setOpaque', (prm_bool ? 'T' : 'F') )
	end
	def yk_send_client_property( prm_key, prm_val )
		k = YKLookComponent.yks_encode_property( prm_key )
		v = YKLookComponent.yks_encode_property( prm_val )
		yks_rpc( 'yk_putClientProperty', k, v )
	end
	def self.yks_encode_property( prm_dat )
		if prm_dat.kind_of?(Integer)
			return ( 'i/' + prm_dat.to_s() )
		end
		if prm_dat.kind_of?(String)
			return ( 'S/' + YKLookMain.yks_encode_string( prm_dat ) )
		end
		if prm_dat.kind_of?(Symbol)
			return ( 'S/' + YKLookMain.yks_encode_string( prm_dat ) )
		end
		if prm_dat.kind_of?(TrueClass)
			return 's/true'
		end
		if prm_dat.kind_of?(FalseClass)
			return 's/false'
		end
		if prm_dat.kind_of?(NilClass)
			return 'n/null'
		end
		raise YKLookErrorSwing, 'illParamObject'
	end
	def yk_request_focus()
		yks_rpc( 'yk_requestFocus' )
	end
end
class YKLookLabel < YKLookComponent
	def initialize( prm_text = nil, prm_image = nil, prm_pos = nil )
		super()
		yk_set_label( prm_text, prm_image, nil, prm_pos )
	end
end
class YKLookButton < YKLookComponent
	def initialize( prm_text = nil, prm_image = nil, prm_pos = nil )
		super()
		yk_set_label( prm_text, prm_image, nil, prm_pos )
	end
end
class YKLookToggleButton < YKLookComponent
	def initialize( prm_text=nil, prm_image=nil, prm_sel=nil, prm_pos=nil )
		super()
		@yk_group = nil
		yk_set_label( prm_text, prm_image, prm_sel, prm_pos )
	end
	def yk_set_selected( prm_flag )
		super( prm_flag )
		if @yk_group
			if prm_flag
				@yk_group.yks_select_on( self )
			else
				yks_set_bool('yk_selected',true)
			end
		end
	end
	def yks_set_group( prm_group )
		@yk_group = prm_group
		if prm_group.yk_selected_button()
			yks_set_bool('yk_selected',false)
		end
	end
	def yks_decode( prm_ary, prm_idx )
		@yk_prop['yk_selected'] = (prm_ary[prm_idx] == "true")
		return super( prm_ary, prm_idx + 1 )
	end
end
class YKLookCheckBox < YKLookToggleButton
	def initialize( prm_text=nil, prm_image=nil, prm_sel=nil, prm_pos=nil )
		super( prm_text, prm_image, prm_sel, prm_pos )
	end
end
class YKLookRadioButton < YKLookToggleButton
	def initialize( prm_text=nil, prm_image=nil, prm_sel=nil, prm_pos=nil )
		super( prm_text, prm_image, prm_sel, prm_pos )
	end
end
class YKLookButtonGroup < YKLookSwingBase
	def initialize( prm_btns = [] )
		if not prm_btns.kind_of?(Array)
			raise YKLookErrorSwing, 'notArray'
		end
		super()
		@yk_buttons = []
		yk_add( prm_btns )
	end
	def yks_select_on( prm_btn )
		@yk_buttons.each() do |btn|
			btn.yks_set_bool('yk_selected', (btn == prm_btn) )
		end
	end
	def yk_buttons()
		return @yk_buttons
	end
	def yk_add( prm_btn = [] )
		if not prm_btn.kind_of?(Array)
			prm_btn = [ prm_btn ]
		end
		if prm_btn.empty?()
			return
		end
		ary = [ prm_btn.size() ]
		prm_btn.each() do |btn|
			if not btn.kind_of?(YKLookToggleButton)
				raise YKLookErrorSwing, 'illParamObject'
			end
			btn.yks_set_group( self )
			ary << btn.yk_id()
			@yk_buttons << btn
		end
		yks_rpca( 'yk_add', ary )
	end
	def yk_selected_index()
		@yk_buttons.each_with_index() do |btn,idx|
			if btn.yk_is_selected()
				return idx
			end
		end
		return -1
	end
	def yk_selected_button()
		@yk_buttons.each() do |btn|
			if btn.yk_is_selected()
				return btn
			end
		end
		return nil
	end
end
class YKLookSelectable < YKLookComponent
	def initialize( prm_data = [] )
		super()
		yks_set_item( prm_data )
	end
	def yks_set_item( prm_data )
		if not prm_data.kind_of?(Array)
			raise YKLookErrorSwing, 'notArray'
		end
		ary = [ prm_data.size() ]
		prm_data.each() do |item|
			ary << yks_encode_item( item )
		end
		yks_rpca( 'yk_setItem', ary )
	end
	def yks_encode_item( prm_item )
		if prm_item.kind_of?(String)
			return 'S/' + YKLookMain.yks_encode_string( prm_item )
		end
		if prm_item.kind_of?(Integer)
			return 'i/' + prm_item.to_s()
		end
		if prm_item.kind_of?(Float)
			return 'd/' + prm_item.to_s()
		end
		if kind_of?(YKLookComboBox)
			raise YKLookErrorSwing, 'illComboObject'
		end
		if not prm_item.kind_of?(YKLookImage)
			yks_check_gui( prm_item )
		end
		return 'o/' + prm_item.yk_id().to_s()
	end
	def yk_set_item( prm_data )
		yks_set_item( prm_data )
	end
	def yk_add_item( prm_data, prm_pos = 999999 )
		if not prm_data.kind_of?(Array)
			prm_data = [ prm_data ]
		end
		ary = [ prm_pos, prm_data.size() ]
		prm_data.each() do |val|
			ary << yks_encode_item( val )
		end
		yks_rpca( 'yk_addItem', ary )
	end
	def yk_remove_at_item( prm_item )
		yks_rpc( 'yk_removeAtItem', yks_encode_item( prm_item ) )
	end
	def yk_remove_at_index( prm_idx )
		yks_rpc( 'yk_removeAtIndex', prm_idx )
	end
	def yk_set_selected_index( prm_idx )
		yks_rpc( 'yk_setSelectedIndex', prm_idx )
	end
	def yk_set_selected_item( prm_item )
		yks_rpc( 'yk_setSelectedItem', yks_encode_item( prm_item ) )
	end
	def yk_set_row( prm_row )
		yks_rpc( 'yk_setRaw', prm_row )
	end
	def yk_selected_index()
		idx = @yk_prop['yk_selected_index']
		if idx
			return idx
		end
		return -1
	end
	def yk_selected_item()
		return yks_decode_item( @yk_prop['yk_selected_item'] )
	end
	def yks_decode_item( prm_item )
		if not prm_item.kind_of?(String)
			return nil
		end
		if prm_item[0,1] == 'S'
			return YKLookMain.yks_decode_string( prm_item[2..-1] )
		end
		if prm_item[0,1] == 'i'
			return prm_item[2..-1].to_i()
		end
		if prm_item[0,1] == 'd'
			return prm_item[2..-1].to_f()
		end
		return YKLookSwingBase.yks_find_source( prm_item[2..-1].to_i() )
	end
	def yks_decode( prm_ary, prm_idx )
		@yk_prop['yk_selected_index'] = prm_ary[prm_idx].to_i()
		@yk_prop['yk_selected_item' ] = prm_ary[prm_idx+1]
		return super( prm_ary, prm_idx + 2 )
	end
end
class YKLookPopupList < YKLookSelectable
end
class YKLookComboBox < YKLookSelectable
end
class YKLookList < YKLookSelectable
	def initialize( prm_data = [] )
		super( prm_data )
		yks_rpc( 'yk_setScrollPane' )
	end
	def yk_replace_item( prm_item, prm_idx )
		yks_rpc( 'yk_replaceItem', prm_idx, yks_encode_item( prm_item ) )
	end
	def yk_scroll_to_index( prm_idx )
		yks_rpc( 'yk_scrollToIndex', prm_idx )
	end
	def yk_scroll_to_item( prm_item )
		yks_rpc( 'yk_scrollToItem', yks_encode_item( prm_item ) )
	end
	def yk_select_none()
		yks_rpc( 'yk_selectNone' )
	end
	def yk_set_selected_index( prm_data )
		if not prm_data.kind_of?(Array)
			prm_data = [ prm_data ]
		end
		yks_rpc( 'yk_setSelectedIndex', prm_data.size(), *prm_data )
	end
	def yk_set_selected_item( prm_data )
		if not prm_data.kind_of?(Array)
			prm_data = [ prm_data ]
		end
		ary = [ prm_data.size() ]
		prm_data.each() do |item|
			ary << yks_encode_item( item )
		end
		yks_rpca( 'yk_setSelectedItem', ary )
	end
	def yk_set_select_mode( prm_sel )
		yks_rpc( 'yk_setSelectMode', prm_sel )
	end
	def yk_set_listener( prm_lis, prm_click = 1 )
		yks_rpc( 'yk_setListener', yks_check_listener( prm_lis ), prm_click )
		return self
	end
	def yk_selected_index()
		sel = @yk_prop['yk_selected_index']
		if sel
			return sel
		end
		return []
	end
	def yk_selected_item()
		sel = @yk_prop['yk_selected_item']
		if sel.kind_of?(Array)
			ary = []
			sel.each() do |val|
				ary << yks_decode_item( val )
			end
			return ary
		end
		return []
	end
	def yks_decode( prm_ary, prm_idx )
		len = prm_ary[prm_idx].to_i()
		prm_idx += 1
		cnt = 0
		selidx = []
		selval = []
		while cnt < len
			selidx << prm_ary[prm_idx  ].to_i()
			selval << prm_ary[prm_idx+1]
			prm_idx += 2
			cnt += 1
		end
		@yk_prop['yk_selected_index'] = selidx
		@yk_prop['yk_selected_item' ] = selval
		return yks_decode_trail( prm_ary, prm_idx )
	end
end
class YKLookTextField < YKLookComponent
	def initialize( prm_col = nil, prm_text = nil )
		super()
		if prm_col
			yks_rpc( 'yk_setColumns', prm_col )
		end
		if prm_text
			yk_set_text( prm_text )
			@yk_select = [ '', '', prm_text ]
		else
			@yk_select = [ '', '', '' ]
		end
	end
	def yk_set_text( prm_text )
		super( prm_text )
		yk_set_caret_position( 0 )
	end
	def yk_set_editable( prm_bool )
		yks_rpc( 'yk_setEditable', prm_bool.kind_of?(TrueClass).to_s() )
	end
	def yk_set_caret_position( prm_pos )
		if prm_pos == 0 or prm_pos == -1
			yks_rpc( 'yk_setCaretPosition', prm_pos )
		end
	end
	def yk_select_all()
		yks_rpc( 'yk_selectAll' )
	end
	def yk_selected_text( prm_index = 1 )
		return @yk_select[prm_index]
	end
	def yks_decode( prm_ary, prm_idx )
		@yk_prop['yk_text'] = YKLookMain.yks_decode_string( prm_ary[prm_idx] )
		s1 = YKLookMain.yks_decode_string( prm_ary[prm_idx+1] )
		s2 = YKLookMain.yks_decode_string( prm_ary[prm_idx+2] )
		s3 = YKLookMain.yks_decode_string( prm_ary[prm_idx+3] )
		@yk_select = [ s1, s2, s3 ]
		return super( prm_ary, prm_idx + 4 )
	end
end
class YKLookPasswordField < YKLookTextField
	def yk_set_echo_char( prm_echo )
		yks_rpc( 'yk_setEchoChar', YKLookMain.yks_encode_string( prm_echo ) )
	end
end
class YKLookTextArea < YKLookTextField
	def initialize( prm_row = nil, prm_col = nil, prm_text = nil )
		super( prm_col, prm_text )
		if prm_row
			yks_rpc( 'yk_setRows', prm_row )
		end
		yk_set_line_wrap( true );
		yks_rpc( 'yk_setScrollPane' )
	end
	def yk_append_text( prm_text )
		yks_rpc( 'yk_append', YKLookMain.yks_encode_string( prm_text ) )
	end
	def yk_set_line_wrap( prm_wrap )
		yks_rpc( 'yk_setLineWrap', prm_wrap.kind_of?(TrueClass).to_s() )
	end
	def yk_set_tab_size( prm_size )
		yks_rpc( 'yk_setTabSize', prm_size )
	end
	def yk_set_wrap_style_word( prm_word )
		yks_rpc( 'yk_setWrapStyleWord', prm_word.kind_of?(TrueClass).to_s() )
	end
end
class YKLookCursor < YKLookSwingBase
	def initialize( prm_any, prm_x = 0, prm_y = 0, prm_name = nil )
		super()
		if prm_any.kind_of?(YKLookImage)
			yks_rpc( 'yk_setImage', prm_any.yk_id(), prm_x, prm_y,
									YKLookMain.yks_encode_string( prm_name ) )
		else
			yks_rpc( 'yk_setPred', 's/' + prm_any.to_s() )
		end
	end
end
class YKLookImage < YKLookSwingBase
	def yks_set_rawdata( prm_raw )
		yks_rpca( 'yk_setRawData', prm_raw.unpack('H*') )
	end
	def yks_set_url( prm_url, prm_file )
		yks_rpca( 'yk_setUrl', YKLookMain.yks_encode_url( prm_url, prm_file ) )
	end
	def self.yk_from_file( prm_path )
		f = File.open( File.expand_path( prm_path ), 'rb' )
		rawdata = f.read()
		f.close()
		img = YKLookImage.new()
		img.yks_set_rawdata( rawdata )
		return img
	end
	def self.yk_from_url( prm_url, prm_file = '' )
		img = YKLookImage.new()
		img.yks_set_url( prm_url, prm_file )
		return img
	end
end
class YKLookColor < YKLookSwingBase
	def initialize( prm_data )
		super()
		yks_rpc( 'yk_setColor', yks_encode_color( prm_data ) )
	end
end
class YKLookFont < YKLookSwingBase
	def initialize( prm_name = nil, prm_style = 'P', prm_size = 12 )
		super()
		if not prm_style.kind_of?(String)
			raise YKLookErrorSwing, 'fontStyleNeedsString'
		end
		if not prm_size.kind_of?(Integer)
			raise YKLookErrorSwing, 'fontSizeNeedsInteger'
		end
		if prm_name.kind_of?(Array)
			nam = nil
			ary = []
			prm_name.each() do |info|
				nam = nil
				sty = nil
				siz = nil
				if info.kind_of?(Array)
					if info.size() >= 1
						if info[0]
							nam = YKLookMain.yks_encode_string( info[0] )
						end
						if info.size() >= 2
							for idx in 1..-1 do
								if info[idx].kind_of?(String)
									sty = info[idx]
								end
								if info[idx].kind_of?(Integer)
									siz = info[idx]
								end
							end
						end
					end
				else
					if info
						nam = YKLookMain.yks_encode_string( info )
					end
				end
				ary << (nam ? nam : 'null')
				ary << (sty ? sty : prm_style)
				ary << 'yk_style_end'
				ary << (siz ? siz : prm_size)
			end
			if nam
				ary << 'null'
				ary << prm_style
				ary << 'yk_style_end'
				ary << prm_size
			end
		else
			if prm_name
				case prm_name.upcase()
					when 'DIALOG','DIALOGINPUT','MONOSPACED',
						'SERIF','SANSSERIF','SYMBOL'
					else raise YKLookErrorSwing, 'fontNameNeedsJavaStd'
				end
				ary = prm_name.unpack('H*')
			else
				ary = ['null']
			end
			ary << prm_style
			ary << 'yk_style_end'
			ary << prm_size
		end
		yks_rpca( 'yk_setFont', ary )
	end
end
class YKLookPanel < YKLookComponent
	def yks_gap( prm_gap )
		if prm_gap.kind_of?(Array) and prm_gap.size() == 2
			if prm_gap[0].kind_of?(Integer)
				yks_rpc( 'yk_setHgap', prm_gap[0] )
			end
			if prm_gap[1].kind_of?(Integer)
				yks_rpc( 'yk_setVgap', prm_gap[1] )
			end
		end
	end
	def yk_add( prm_gui )
		yks_check_gui( prm_gui )
		yks_rpc( 'yk_add', prm_gui.yk_id() )
	end
end
class YKLookPanelFlow < YKLookPanel
	def initialize( prm_align = nil, prm_gap = nil )
		super()
		if prm_align
			yks_rpc( 'yk_setAlignment', prm_align )
		end
		yks_gap( prm_gap )
		yks_rpc( 'yk_setLayout' )
	end
end
class YKLookPanelBox < YKLookPanel
	def initialize( prm_axis )
		super()
		yks_rpc( 'yk_setAxis', prm_axis )
		yks_rpc( 'yk_setLayout' )
	end
end
class YKLookPanelGrid < YKLookPanel
	def initialize( prm_rows, prm_cols, prm_gap = nil )
		super()
		yks_rpc( 'yk_setGrid', prm_rows, prm_cols )
		yks_gap( prm_gap )
		yks_rpc( 'yk_setLayout' )
	end
end
class YKLookPanelGridBag < YKLookPanel
	def initialize()
		super()
		yks_rpc( 'yk_setLayout' )
	end
	def yk_add( prm_gui, prm_rect, prm_fill= 'N', prm_geom= nil, prm_acr= 'C' )
		yks_check_gui( prm_gui )
		ary = [ prm_gui.yk_id() ]
		if prm_rect.kind_of?(Hash)
			gx = prm_rect['gridx']
			gy = prm_rect['gridy']
			gw = prm_rect.key?('gridwidth') ? prm_rect['gridwidth'] : 1
			gh = prm_rect.key?('gridheight') ? prm_rect['gridheight'] : 1
			it = prm_rect['top']
			il = prm_rect['left']
			ib = prm_rect['bottom']
			ir = prm_rect['right']
			ix = prm_rect['ipadx']
			iy = prm_rect['ipady']
			wx = prm_rect['weightx']
			wy = prm_rect['weighty']
			prm_fill = prm_rect['fill']
			prm_acr  = prm_rect['anchor']
			prm_geom = [ it, il, ib, ir, ix, iy, wx, wy ]
			prm_rect = [ gx, gy, gw, gh ]
		end
		if prm_rect.size() < 4
			prm_rect = [ nil, nil, prm_rect[0], prm_rect[1] ]
		end
		ary << (prm_rect[0] ? prm_rect[0] : 'REL')
		ary << (prm_rect[1] ? prm_rect[1] : 'REL')
		ary << (prm_rect[2] ? prm_rect[2] : 'RMD')
		ary << (prm_rect[3] ? prm_rect[3] : 'RMD')
		ary << (prm_fill    ? prm_fill   : 'N'  )
		if prm_geom.nil?()
			prm_geom = [ 0, 0, 0, 0 ]
		end
		ary << (prm_geom[0] ? prm_geom[0] : 0)
		ary << (prm_geom[1] ? prm_geom[1] : 0)
		ary << (prm_geom[2] ? prm_geom[2] : 0)
		ary << (prm_geom[3] ? prm_geom[3] : 0)
		if prm_geom.size() >= 6
			ary << (prm_geom[4] ? prm_geom[4] : 0)
			ary << (prm_geom[5] ? prm_geom[5] : 0)
		else
			ary << 0
			ary << 0
		end
		if prm_geom.size() >= 8
			ary << (prm_geom[6] ? prm_geom[6] : 'DEF')
			ary << (prm_geom[7] ? prm_geom[7] : 'DEF')
		else
			ary << 'DEF'
			ary << 'DEF'
		end
		ary << (prm_acr ? prm_acr : 'C')
		yks_rpca( 'yk_add', ary )
	end
end
class YKLookPanelCard < YKLookPanel
	def initialize( prm_gap = nil )
		super()
		yks_gap( prm_gap )
		yks_rpc( 'yk_setLayout' )
	end
	def yk_add( prm_gui, prm_key )
		yks_check_gui( prm_gui )
		yks_rpc( 'yk_add', prm_gui.yk_id(),
					YKLookMain.yks_encode_string( prm_key ) )
	end
	def yk_show( prm_key )
		yks_rpc( 'yk_show', YKLookMain.yks_encode_string( prm_key ) )
	end
	def yk_first()
		yks_rpc( 'yk_first' )
	end
	def yk_last()
		yks_rpc( 'yk_last' )
	end
	def yk_next()
		yks_rpc( 'yk_next' )
	end
	def yk_prev()
		yks_rpc( 'yk_previous' )
	end
end
class YKLookPanelQuint < YKLookPanel
	def initialize( prm_gap = nil )
		super()
		yks_gap( prm_gap )
		yks_rpc( 'yk_setLayout' )
	end
	def yk_add( prm_gui, prm_pos = 'c' )
		yks_check_gui( prm_gui )
		yks_rpc( 'yk_add', prm_gui.yk_id(), prm_pos )
	end
end
class YKLookBoxFiller < YKLookSwingBase
	def initialize( prm_width = nil, prm_height = nil, prm_max = nil )
		super()
		if prm_width.kind_of?(Array)
			yks_rpc('yk_createBoxFiller', prm_width[0], prm_width[1],
						prm_height[0], prm_height[1], prm_max[0], prm_max[1] )
			return
		end
		if prm_width.nil?()
			if prm_height.nil?()
				yks_rpc( 'yk_createGlue' )
			else
				yks_rpc( 'yk_createHorizontalGlue' )
			end
		else
			if prm_height.nil?()
				yks_rpc( 'yk_createVerticalGlue' )
			else
				yks_rpc( 'yk_createRigidArea', prm_width, prm_height )
			end
		end
	end
end
class YKLookEvent
	def initialize()
		super()
		@yk_source   = nil
		@yk_frameid  = 0
		@yk_callback = nil
		@yk_modkey   = []
		@yk_button   = []
	end
	def yk_source()
		return @yk_source
	end
	def yks_frame_id()
		return @yk_frameid
	end
	def yk_callback()
		return @yk_callback
	end
	def yk_modkey()
		return @yk_modkey
	end
	def yk_button()
		return @yk_button
	end
	def yks_decode( prm_ary )
		id = prm_ary[1].to_i()
		if id == 0
			@yk_source = YKLookMain.yk_main()
		else
			@yk_source = YKLookSwingBase.yks_find_source( id )
		end
		@yk_frameid  = prm_ary[2].to_i()
		@yk_callback = YKLookMain.yks_decode_string( prm_ary[3] )
	end
	def yks_decode_modifiers( prm_ary )
		prm_ary.each() do |item|
			case item
				when 'SHIFT'     then @yk_modkey << item
				when 'CTRL'      then @yk_modkey << item
				when 'META'      then @yk_modkey << item
				when 'ALT'       then @yk_modkey << item
				when 'ALT_GRAPH' then @yk_modkey << item
				when 'BUTTON1'   then @yk_button << item
				when 'BUTTON2'   then @yk_button << item
				when 'BUTTON3'   then @yk_button << item
			end
		end
	end
	def yks_decode_values( prm_vals )
		ary = []
		typ = nil
		prm_vals.each_with_index() do |val,idx|
			if idx % 2 == 0
				typ = val
			else
				case typ
					when 'i' then dat = val.to_i()
					when 's' then dat = val
					when 'S' then dat = YKLookMain.yks_decode_string( val )
					else          dat = nil
				end
				ary << dat
			end
		end
		return ary
	end
end
class YKLookEventDialog < YKLookEvent
	def yk_answer( prm_idx = 0 )
		if prm_idx.nil?()
			return @yk_answer
		end
		if prm_idx < @yk_answer.size()
			return @yk_answer[prm_idx]
		end
		return nil
	end
	def yks_decode( prm_ary )
		super( prm_ary )
		@yk_answer = yks_decode_values( prm_ary[4..-1] )
	end
end
class YKLookEventAction < YKLookEvent
	def yks_decode( prm_ary )
		super( prm_ary )
		yks_decode_modifiers( prm_ary[4..-1] )
	end
end
class YKLookEventListClick < YKLookEvent
	def yks_decode( prm_ary )
		super( prm_ary )
		yks_decode_modifiers( prm_ary[4..-1] )
	end
end
class YKLookEventFrame < YKLookEventDialog
end

YKLookLog.yks_cwd()
YKLookMain.yks_stdio_redirect()
# end
